@@ -60,8 +60,8 @@ struct _IndicateIndicatorPrivate
 G_DEFINE_TYPE (IndicateIndicator, indicate_indicator, G_TYPE_OBJECT);
 
 static void indicate_indicator_finalize (GObject * object);
-static void set_property (IndicateIndicator * indicator, const gchar * key, const gchar * data);
-static const gchar * get_property (IndicateIndicator * indicator, const gchar * key);
+static void set_property (IndicateIndicator * indicator, const gchar * key, const GValue * data);
+static const GValue * get_property (IndicateIndicator * indicator, const gchar * key);
 static GPtrArray * list_properties (IndicateIndicator * indicator);
 
 
@@ -91,8 +91,8 @@ indicate_indicator_class_init (IndicateIndicatorClass * class)
 	                                     G_SIGNAL_RUN_LAST,
 	                                     G_STRUCT_OFFSET(IndicateIndicatorClass, user_display),
 	                                     NULL, NULL,
-	                                     g_cclosure_marshal_VOID__VOID,
-	                                     G_TYPE_NONE, 0);
+	                                     g_cclosure_marshal_VOID__UINT,
+	                                     G_TYPE_NONE, 1, G_TYPE_UINT);
 	/**
 		IndicateIndicator::hide:
 		@arg0: The #IndicateIndicator object
@@ -166,6 +166,18 @@ indicate_indicator_class_init (IndicateIndicatorClass * class)
 	return;
 }
 
+/* A small little function to both clear the insides of a 
+   value as well as the memory it itself uses. */
+static void 
+_g_value_free (gpointer data)
+{
+	if (data == NULL) return;
+	GValue * value = (GValue*)data;
+	g_value_unset(value);
+	g_free(data);
+	return;
+}
+
 static void
 indicate_indicator_init (IndicateIndicator * indicator)
 {
@@ -175,7 +187,7 @@ indicate_indicator_init (IndicateIndicator * indicator)
 	priv->is_visible = FALSE;
 	priv->is_displayed = FALSE;
 
-	priv->properties = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+	priv->properties = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, _g_value_free);
 
 	priv->server = indicate_server_ref_default();
 	priv->id = indicate_server_get_next_id(priv->server);
@@ -358,23 +370,41 @@ indicate_indicator_get_id (IndicateIndicator * indicator)
 	return priv->id;
 }
 
+/**
+	indicate_indicator_get_server:
+	@indicator: a #IndicateIndicator to act on
+
+	Gets the server of the @indicator.
+
+	Return value: The server associated with the indicator
+		or NULL if error.
+*/
+IndicateServer *
+indicate_indicator_get_server (IndicateIndicator * indicator)
+{
+	g_return_val_if_fail(INDICATE_IS_INDICATOR(indicator), NULL);
+	IndicateIndicatorPrivate * priv = INDICATE_INDICATOR_GET_PRIVATE(indicator);
+	return priv->server;
+}
+
 /**
 	indicate_indicator_user_display:
 	@indicator: a #IndicateIndicator to act on
+	@timestamp: The time that the event happened
 
 	Emits the #IndicateIndicator::user-display signal simliar to a user
 	clicking on @indicator over the bus.  Signal will not be sent if the
 	@indicator is not visible.
 */
 void
-indicate_indicator_user_display (IndicateIndicator * indicator)
+indicate_indicator_user_display (IndicateIndicator * indicator, guint timestamp)
 {
 	IndicateIndicatorPrivate * priv = INDICATE_INDICATOR_GET_PRIVATE(indicator);
 	if (!priv->is_visible) {
 		return;
 	}
 
-	g_signal_emit(indicator, signals[USER_DISPLAY], 0, TRUE);
+	g_signal_emit(indicator, signals[USER_DISPLAY], 0, timestamp, TRUE);
 	return;
 }
 
@@ -398,7 +428,11 @@ indicate_indicator_set_property (IndicateIndicator * indicator, const gchar * ke
 		return;
 	}
 
-	return class->set_property(indicator, key, data);
+	GValue strval = {0};
+	g_value_init(&strval, G_TYPE_STRING);
+	g_value_set_static_string(&strval, data);
+
+	return class->set_property(indicator, key, &strval);
 }
 
 /**
@@ -436,12 +470,16 @@ indicate_indicator_set_property_time (IndicateIndicator * indicator, const gchar
 void
 indicate_indicator_set_property_int (IndicateIndicator * indicator, const gchar * key, gint value)
 {
-	gchar * valuestr = g_strdup_printf("%d", value);
-	if (valuestr != NULL) {
-		indicate_indicator_set_property(indicator, key, valuestr);
+	IndicateIndicatorClass * class = INDICATE_INDICATOR_GET_CLASS(indicator);
+	if (class->set_property == NULL) {
+		return;
 	}
-	g_free(valuestr);
-	return;
+
+	GValue intval = {0};
+	g_value_init(&intval, G_TYPE_INT);
+	g_value_set_int(&intval, value);
+
+	return class->set_property(indicator, key, &intval);
 }
 
 /**
@@ -457,12 +495,27 @@ indicate_indicator_set_property_int (IndicateIndicator * indicator, const gchar
 void
 indicate_indicator_set_property_bool (IndicateIndicator * indicator, const gchar * key, gboolean value)
 {
-	if (value) {
-		indicate_indicator_set_property(indicator, key, INDICATE_INDICATOR_VALUE_TRUE);
-	} else {
-		indicate_indicator_set_property(indicator, key, INDICATE_INDICATOR_VALUE_FALSE);
+	IndicateIndicatorClass * class = INDICATE_INDICATOR_GET_CLASS(indicator);
+	if (class->set_property == NULL) {
+		return;
 	}
-	return;
+
+	GValue boolval = {0};
+	g_value_init(&boolval, G_TYPE_BOOLEAN);
+	g_value_set_boolean(&boolval, value);
+
+	return class->set_property(indicator, key, &boolval);
+}
+
+void
+indicate_indicator_set_property_value (IndicateIndicator * indicator, const gchar * key, GValue * value)
+{
+	IndicateIndicatorClass * class = INDICATE_INDICATOR_GET_CLASS(indicator);
+	if (class->set_property == NULL) {
+		return;
+	}
+
+	return class->set_property(indicator, key, value);
 }
 
 /**
@@ -483,6 +536,18 @@ indicate_indicator_get_property (IndicateIndicator * indicator, const gchar * ke
 		return NULL;
 	}
 
+	const GValue * val = class->get_property(indicator, key);
+	return g_value_get_string(val);
+}
+
+const GValue *
+indicate_indicator_get_property_value (IndicateIndicator * indicator, const gchar * key)
+{
+	IndicateIndicatorClass * class = INDICATE_INDICATOR_GET_CLASS(indicator);
+	if (class->get_property == NULL) {
+		return NULL;
+	}
+
 	return class->get_property(indicator, key);
 }
 
@@ -508,36 +573,35 @@ indicate_indicator_list_properties (IndicateIndicator * indicator)
 }
 
 static void
-set_property (IndicateIndicator * indicator, const gchar * key, const gchar * data)
+set_property (IndicateIndicator * indicator, const gchar * key, const GValue * data)
 {
 	g_return_if_fail(INDICATE_IS_INDICATOR(indicator));
 
 	IndicateIndicatorPrivate * priv = INDICATE_INDICATOR_GET_PRIVATE(indicator);
 
-	gchar * current = g_hash_table_lookup(priv->properties, key);
-	if (current == NULL || g_strcmp0(current, data)) {
-		/* If the value has changed or there is no value */
-		gchar * newkey = g_strdup(key);
-		/* g_debug("What is newkey? %s", newkey); */
-		g_hash_table_insert(priv->properties, newkey, g_strdup(data));
-		if (indicate_indicator_is_visible(indicator)) {
-			/* g_debug("Indicator property modified: %s %s", key, data); */
-			g_signal_emit(indicator, signals[MODIFIED], 0, key, TRUE);
-		}
+	gchar * newkey = g_strdup(key);
+	GValue * newval = g_new0(GValue, 1);
+	g_value_init(newval, G_VALUE_TYPE(data));
+	g_value_copy(data, newval);
+
+	g_hash_table_replace(priv->properties, newkey, newval);
+
+	if (indicate_indicator_is_visible(indicator)) {
+		/* g_debug("Indicator property modified: %s", key); */
+		g_signal_emit(indicator, signals[MODIFIED], 0, key, TRUE);
 	}
 
 	return;
 }
 
-static const gchar *
+static const GValue *
 get_property (IndicateIndicator * indicator, const gchar * key)
 {
 	g_return_val_if_fail(INDICATE_IS_INDICATOR(indicator), NULL);
 
 	IndicateIndicatorPrivate * priv = INDICATE_INDICATOR_GET_PRIVATE(indicator);
 
-	// TODO: Think about whether we should be strdup'ing this.  Seems like overkill, but might not be.
-	return (const gchar *)g_hash_table_lookup(priv->properties, key);
+	return (const GValue *)g_hash_table_lookup(priv->properties, key);
 }
 
 static GPtrArray *
