@@ -29,9 +29,11 @@ License version 3 and version 2.1 along with this program.  If not, see
  
 #include "server.h"
 #include "interests-priv.h"
-#include "server-marshal.h"
+#include "indicate-marshal.h"
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
+#include <libdbusmenu-glib/server.h>
+#include "dbus-shared.h"
 
 /* Errors */
 enum {
@@ -50,6 +52,7 @@ enum {
 	REMOVE_INTEREST_FAILED,
 	NO_MAX_INDICATORS_SET,
 	MAX_INDICATORS_SET_FAILED,
+	NO_SUCH_PROPERTY,
 	LAST_ERROR
 };
 
@@ -74,7 +77,8 @@ enum {
 	PROP_0,
 	PROP_DESKTOP,
 	PROP_TYPE,
-	PROP_COUNT
+	PROP_COUNT,
+	PROP_MENU
 };
 
 static guint signals[LAST_SIGNAL] = { 0 };
@@ -96,6 +100,8 @@ struct _IndicateServerPrivate
 	gchar * type;
 	guint count;
 
+	DbusmenuServer * dbusmenu;
+
 	// TODO: Should have a more robust way to track this, but this'll work for now
 	guint num_hidden;
 
@@ -125,14 +131,16 @@ static const gint MAX_INDICATORS_UNSET = -2;
 G_DEFINE_TYPE (IndicateServer, indicate_server, G_TYPE_OBJECT);
 
 /* Prototypes */
+static DBusHandlerResult dbus_filter_new_listener (DBusConnection * connection, DBusMessage * message, void * user_data);
+static void indicate_server_dispose (GObject * obj);
 static void indicate_server_finalize (GObject * obj);
 static gboolean get_indicator_count (IndicateServer * server, guint * count, GError **error);
 static gboolean get_indicator_list (IndicateServer * server, GArray ** indicators, GError ** error);
 static IndicateIndicator * get_indicator (IndicateServer * server, guint id, GError **error);
-static gboolean get_indicator_property (IndicateServer * server, guint id, gchar * property, gchar ** value, GError **error);
+static gboolean get_indicator_property (IndicateServer * server, guint id, gchar * property, GValue * value, GError **error);
 static gboolean get_indicator_property_group (IndicateServer * server, guint id, GPtrArray * properties, gchar *** value, GError **error);
 static gboolean get_indicator_properties (IndicateServer * server, guint id, gchar *** properties, GError **error);
-static gboolean show_indicator_to_user (IndicateServer * server, guint id, GError ** error);
+static gboolean show_indicator_to_user (IndicateServer * server, guint id, guint timestamp, GError ** error);
 static gboolean indicator_displayed (IndicateServer * server, gchar * sender, guint id, gboolean displayed, GError ** error);
 static void dbus_owner_change (DBusGProxy * proxy, const gchar * name, const gchar * prev, const gchar * new, IndicateServer * server);
 static void indicator_display_check_recalc (gpointer key, gpointer value, gpointer userdata);
@@ -153,19 +161,19 @@ static void indicate_server_interested_folks_copy (IndicateServerInterestedFolk
 static void indicate_server_interested_folks_destroy(IndicateServerInterestedFolk * folk);
 
 /* DBus API */
-gboolean _indicate_server_get_indicator_count (IndicateServer * server, guint * count, GError **error);
-gboolean _indicate_server_get_indicator_list (IndicateServer * server, GArray ** indicators, GError ** error);
-gboolean _indicate_server_get_indicator_property (IndicateServer * server, guint id, gchar * property, gchar ** value, GError **error);
-gboolean _indicate_server_get_indicator_property_group (IndicateServer * server, guint id, GPtrArray * properties, gchar *** value, GError **error);
-gboolean _indicate_server_get_indicator_properties (IndicateServer * server, guint id, gchar *** properties, GError **error);
-gboolean _indicate_server_show_indicator_to_user (IndicateServer * server, guint id, GError ** error);
-gboolean _indicate_server_indicator_displayed (IndicateServer * server, guint id, gboolean displayed, DBusGMethodInvocation * method);
-gboolean _indicate_server_show_interest (IndicateServer * server, gchar * interest, DBusGMethodInvocation * method);
-gboolean _indicate_server_remove_interest (IndicateServer * server, gchar * interest, DBusGMethodInvocation * method);
-gboolean _indicate_server_set_max_indicators (IndicateServer * server, gint max, DBusGMethodInvocation * method);
+gboolean _indicate_interface_server_get_indicator_count (IndicateServer * server, guint * count, GError **error);
+gboolean _indicate_interface_server_get_indicator_list (IndicateServer * server, GArray ** indicators, GError ** error);
+gboolean _indicate_interface_server_get_indicator_property (IndicateServer * server, guint id, gchar * property, GValue * value, GError **error);
+gboolean _indicate_interface_server_get_indicator_property_group (IndicateServer * server, guint id, GPtrArray * properties, gchar *** value, GError **error);
+gboolean _indicate_interface_server_get_indicator_properties (IndicateServer * server, guint id, gchar *** properties, GError **error);
+gboolean _indicate_interface_server_show_indicator_to_user (IndicateServer * server, guint id, guint timestamp, GError ** error);
+gboolean _indicate_interface_server_indicator_displayed (IndicateServer * server, guint id, gboolean displayed, DBusGMethodInvocation * method);
+gboolean _indicate_interface_server_show_interest (IndicateServer * server, gchar * interest, DBusGMethodInvocation * method);
+gboolean _indicate_interface_server_remove_interest (IndicateServer * server, gchar * interest, DBusGMethodInvocation * method);
+gboolean _indicate_interface_server_set_max_indicators (IndicateServer * server, gint max, DBusGMethodInvocation * method);
 
 /* Has to be after the dbus prototypes */
-#include "dbus-indicate-server.h"
+#include "indicate-interface-server.h"
 
 
 
@@ -179,6 +187,7 @@ indicate_server_class_init (IndicateServerClass * class)
 
 	g_type_class_add_private (class, sizeof (IndicateServerPrivate));
 
+	gobj->dispose = indicate_server_dispose;
 	gobj->finalize = indicate_server_finalize;
 	gobj->set_property = set_property;
 	gobj->get_property = get_property;
@@ -196,7 +205,7 @@ indicate_server_class_init (IndicateServerClass * class)
 	                                        G_SIGNAL_RUN_LAST,
 	                                        G_STRUCT_OFFSET (IndicateServerClass, indicator_added),
 	                                        NULL, NULL,
-	                                        _indicate_server_marshal_VOID__UINT,
+	                                        _indicate_marshal_VOID__UINT,
 	                                        G_TYPE_NONE, 1, G_TYPE_UINT);
 	/**
 		IndicateServer::indicator-removed:
@@ -211,7 +220,7 @@ indicate_server_class_init (IndicateServerClass * class)
 	                                        G_SIGNAL_RUN_LAST,
 	                                        G_STRUCT_OFFSET (IndicateServerClass, indicator_removed),
 	                                        NULL, NULL,
-	                                        _indicate_server_marshal_VOID__UINT,
+	                                        _indicate_marshal_VOID__UINT,
 	                                        G_TYPE_NONE, 1, G_TYPE_UINT);
 	/**
 		IndicateServer::indicator-modified:
@@ -227,7 +236,7 @@ indicate_server_class_init (IndicateServerClass * class)
 	                                        G_SIGNAL_RUN_LAST,
 	                                        G_STRUCT_OFFSET (IndicateServerClass, indicator_modified),
 	                                        NULL, NULL,
-	                                        _indicate_server_marshal_VOID__UINT_STRING,
+	                                        _indicate_marshal_VOID__UINT_STRING,
 	                                        G_TYPE_NONE, 2, G_TYPE_UINT, G_TYPE_STRING);
 	/**
 		IndicateServer::server-show:
@@ -263,6 +272,7 @@ indicate_server_class_init (IndicateServerClass * class)
 	/**
 		IndicateServer::server-display:
 		@arg0: The #IndicateServer object
+		@arg1: Timestamp of the show event
 
 		Emitted when a listener signals that the server itself should be
 		displayed.  This signal is caused by a user clicking on the application
@@ -273,8 +283,8 @@ indicate_server_class_init (IndicateServerClass * class)
 	                                        G_SIGNAL_RUN_LAST,
 	                                        G_STRUCT_OFFSET (IndicateServerClass, server_display),
 	                                        NULL, NULL,
-	                                        g_cclosure_marshal_VOID__VOID,
-	                                        G_TYPE_NONE, 0);
+	                                        g_cclosure_marshal_VOID__UINT,
+	                                        G_TYPE_NONE, 1, G_TYPE_UINT);
 	/**
 		IndicateServer::interest-added:
 		@arg0: The #IndicateServer object
@@ -351,9 +361,14 @@ indicate_server_class_init (IndicateServerClass * class)
 	                                              "A number reprsenting the number of items in a server",
 												  0, G_MAXUINT, 0,
 	                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	g_object_class_install_property (gobj, PROP_MENU,
+	                                 g_param_spec_boxed("menu", "DBus Menu Object Path",
+	                                              "The DBus Object path to an object with a dbusmenu interface on it.",
+												  DBUS_TYPE_G_OBJECT_PATH,
+	                                              G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	dbus_g_object_type_install_info(INDICATE_TYPE_SERVER,
-	                                &dbus_glib__indicate_server_object_info);
+	                                &dbus_glib__indicate_interface_server_object_info);
 
 	class->get_indicator_count = get_indicator_count;
 	class->get_indicator_list = get_indicator_list;
@@ -379,7 +394,7 @@ indicate_server_init (IndicateServer * server)
 
 	IndicateServerPrivate * priv = INDICATE_SERVER_GET_PRIVATE(server);
 
-	priv->path = g_strdup("/org/freedesktop/indicate");
+	priv->path = g_strdup("/org/ayatana/indicate");
 	priv->indicators = NULL;
 	priv->num_hidden = 0;
 	priv->visible = FALSE;
@@ -388,6 +403,7 @@ indicate_server_init (IndicateServer * server)
 	priv->type = NULL;
 	priv->desktop = NULL;
 	priv->count = 0;
+	priv->dbusmenu = NULL;
 
 	guint i;
 	for (i = INDICATE_INTEREST_NONE; i < INDICATE_INTEREST_LAST; i++) {
@@ -396,6 +412,29 @@ indicate_server_init (IndicateServer * server)
 	priv->interestedfolks = NULL;
 	priv->max_indicators = MAX_INDICATORS_UNSET;
 
+	priv->connection = dbus_g_bus_get(DBUS_BUS_SESSION, NULL);
+
+	dbus_connection_add_filter(dbus_g_connection_get_connection(priv->connection), dbus_filter_new_listener, server, NULL);
+	dbus_bus_add_match(dbus_g_connection_get_connection(priv->connection), "type='signal',interface='" INDICATE_LISTENER_DBUS_IFACE "',member='IndicatorServersReport'", NULL);
+
+	return;
+}
+
+static void
+indicate_server_dispose (GObject * obj)
+{
+	IndicateServer * server = INDICATE_SERVER(obj);
+	IndicateServerPrivate * priv = INDICATE_SERVER_GET_PRIVATE(server);
+
+	if (priv->dbusmenu != NULL) {
+		g_object_unref(priv->dbusmenu);
+		priv->dbusmenu = NULL;
+	}
+
+	if (priv->visible) {
+		g_signal_emit(server, signals[SERVER_HIDE], 0, priv->type ? priv->type : "", TRUE);
+	}
+
 	return;
 }
 
@@ -405,11 +444,6 @@ indicate_server_finalize (GObject * obj)
 	IndicateServer * server = INDICATE_SERVER(obj);
 	IndicateServerPrivate * priv = INDICATE_SERVER_GET_PRIVATE(server);
 
-	/* TODO: This probably shouldn't be as far down as finalize, but it's fine here. */
-	if (priv->visible) {
-		g_signal_emit(server, signals[SERVER_HIDE], 0, priv->type ? priv->type : "", TRUE);
-	}
-
 	if (priv->path) {
 		g_free(priv->path);
 	}
@@ -420,6 +454,8 @@ indicate_server_finalize (GObject * obj)
 		g_free(priv->type);
 	}
 
+	dbus_connection_remove_filter(dbus_g_connection_get_connection(priv->connection), dbus_filter_new_listener, server);
+
 	G_OBJECT_CLASS (indicate_server_parent_class)->finalize (obj);
 
 	return;
@@ -429,7 +465,6 @@ static void
 set_property (GObject * obj, guint id, const GValue * value, GParamSpec * pspec)
 {
 	g_return_if_fail(G_VALUE_HOLDS_STRING(value) || G_VALUE_HOLDS_UINT(value));
-	g_return_if_fail(id == PROP_DESKTOP || id == PROP_TYPE || id == PROP_COUNT);
 
 	IndicateServerPrivate * priv = INDICATE_SERVER_GET_PRIVATE(obj);
 	switch (id) {
@@ -453,6 +488,9 @@ set_property (GObject * obj, guint id, const GValue * value, GParamSpec * pspec)
 		}
 		break;
 	}
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID(obj, id, pspec);
+		break;
 	}
 
 	return;
@@ -463,8 +501,6 @@ set_property (GObject * obj, guint id, const GValue * value, GParamSpec * pspec)
 static void
 get_property (GObject * obj, guint id, GValue * value, GParamSpec * pspec)
 {
-	g_return_if_fail(id == PROP_DESKTOP || id == PROP_TYPE || id == PROP_COUNT);
-
 	IndicateServerPrivate * priv = INDICATE_SERVER_GET_PRIVATE(obj);
 	switch (id) {
 	case PROP_DESKTOP:
@@ -484,11 +520,50 @@ get_property (GObject * obj, guint id, GValue * value, GParamSpec * pspec)
 	case PROP_COUNT:
 		g_value_set_uint(value, priv->count);
 		break;
+	case PROP_MENU:
+		if (priv->dbusmenu != NULL) {
+			GValue strvalue = {0};
+			g_value_init(&strvalue, G_TYPE_STRING);
+			g_object_get_property(G_OBJECT(priv->dbusmenu), DBUSMENU_SERVER_PROP_DBUS_OBJECT, &strvalue);
+			if (g_value_get_string(&strvalue) != NULL) {
+				g_value_set_boxed(value, g_value_dup_string(&strvalue));
+			} else {
+				g_value_set_boxed(value, g_strdup("/"));
+			}
+			g_value_unset(&strvalue);
+		} else {
+			g_value_set_boxed(value, g_strdup("/"));
+		}
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID(obj, id, pspec);
+		break;
 	}
 
 	return;
 }
 
+/* A small dbus filter that waits on the IndicatorServersReport
+   signal and sends the Show signal if this server is not hidden. */
+static DBusHandlerResult
+dbus_filter_new_listener (DBusConnection * connection, DBusMessage * message, void * user_data)
+{
+	if (!dbus_message_is_signal(message, INDICATE_LISTENER_DBUS_IFACE, "IndicatorServersReport")) {
+		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+	}
+
+	IndicateServer * server = INDICATE_SERVER(user_data);
+
+	if (server != NULL) {
+		IndicateServerPrivate * priv = INDICATE_SERVER_GET_PRIVATE(server);
+		if (priv->visible) {
+			g_signal_emit(server, signals[SERVER_SHOW], 0, priv->type ? priv->type : "", TRUE);
+		}
+	}
+
+	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+}
+
 /* Small little function to get an error quark for usage
    with the GError errors back across DBus */
 static GQuark
@@ -594,11 +669,6 @@ indicate_server_hide (IndicateServer * server)
 		priv->dbus_proxy = NULL;
 	}
 
-	if (priv->connection != NULL) {
-		dbus_g_connection_unref (priv->connection);
-		priv->connection = NULL;
-	}
-	
 	return;
 }
 
@@ -1070,7 +1140,7 @@ indicate_server_set_count (IndicateServer * server, guint count)
 	return;
 }
 
-static IndicateServer * default_indicate_server = NULL;
+static IndicateServer * default_indicate_interface_server = NULL;
 
 /**
 	indicate_server_ref_default:
@@ -1086,15 +1156,15 @@ static IndicateServer * default_indicate_server = NULL;
 IndicateServer *
 indicate_server_ref_default (void)
 {
-	if (default_indicate_server != NULL) {
-		g_object_ref(default_indicate_server);
+	if (default_indicate_interface_server != NULL) {
+		g_object_ref(default_indicate_interface_server);
 	} else {
-		default_indicate_server = g_object_new(INDICATE_TYPE_SERVER, NULL);
-		g_object_add_weak_pointer(G_OBJECT(default_indicate_server),
-		                          (gpointer *)&default_indicate_server);
+		default_indicate_interface_server = g_object_new(INDICATE_TYPE_SERVER, NULL);
+		g_object_add_weak_pointer(G_OBJECT(default_indicate_interface_server),
+		                          (gpointer *)&default_indicate_interface_server);
 	}
 
-	return default_indicate_server;
+	return default_indicate_interface_server;
 }
 
 /**
@@ -1110,14 +1180,14 @@ indicate_server_ref_default (void)
 void
 indicate_server_set_default (IndicateServer * server)
 {
-	if (default_indicate_server != NULL) {
+	if (default_indicate_interface_server != NULL) {
 		g_warning("Setting a default Indicator Server when one has already been created.  I'm not going to destroy that one, but let it live.  This may create some odd results if you don't know what you're doing.");
 	}
 
 	if (server != NULL) {
-		default_indicate_server = server;
-		g_object_add_weak_pointer(G_OBJECT(default_indicate_server),
-		                          (gpointer *)&default_indicate_server);
+		default_indicate_interface_server = server;
+		g_object_add_weak_pointer(G_OBJECT(default_indicate_interface_server),
+		                          (gpointer *)&default_indicate_interface_server);
 	}
 
 	return;
@@ -1187,14 +1257,27 @@ get_indicator (IndicateServer * server, guint id, GError **error)
 }
 
 static gboolean
-get_indicator_property (IndicateServer * server, guint id, gchar * property, gchar ** value, GError **error)
+get_indicator_property (IndicateServer * server, guint id, gchar * property, GValue * value, GError **error)
 {
 	IndicateIndicator * indicator = get_indicator(server, id, error);
 	if (indicator == NULL) {
 		return FALSE;
 	}
 
-	*value = g_strdup(indicate_indicator_get_property(indicator, property));
+	const GValue * ind_property = indicate_indicator_get_property_value(indicator, property);
+	if (ind_property == NULL) {
+		if (error != NULL) {
+			g_set_error(error,
+			            indicate_server_error_quark(),
+			            NO_SUCH_PROPERTY,
+			            "Indicator %d has no property named '%s'",
+			            id, property);
+		}
+		return FALSE;
+	}
+
+	g_value_init(value, G_VALUE_TYPE(ind_property));
+	g_value_copy(ind_property, value);
 	return TRUE;
 }
 
@@ -1209,12 +1292,17 @@ get_indicator_property_group (IndicateServer * server, guint id, GPtrArray * pro
 	GPtrArray * array = g_ptr_array_new();
 	int i;
 	for (i = 0; i < properties->len; i++) {
-		const gchar * val = indicate_indicator_get_property(indicator, g_ptr_array_index(properties, i));
-		if (val != NULL) {
-			g_ptr_array_add(array, g_strdup(val));
-		} else {
-			g_ptr_array_add(array, g_strdup(""));
+		const GValue * ind_property = indicate_indicator_get_property_value(indicator, g_ptr_array_index(properties, i));
+
+		if (ind_property == NULL) {
+			continue;
 		}
+
+		GValue * value = g_new0(GValue, 1);
+		g_value_init(value, G_VALUE_TYPE(ind_property));
+		g_value_copy(ind_property, value);
+
+		g_ptr_array_add(array, value);
 	}
 	g_ptr_array_add(array, NULL);
 	*value = (gchar **)g_ptr_array_free(array, FALSE);
@@ -1239,10 +1327,10 @@ get_indicator_properties (IndicateServer * server, guint id, gchar *** propertie
 }
 
 static gboolean
-show_indicator_to_user (IndicateServer * server, guint id, GError ** error)
+show_indicator_to_user (IndicateServer * server, guint id, guint timestamp, GError ** error)
 {
 	if (id == INDICATE_SERVER_INDICATOR_NULL) {
-		g_signal_emit(server, signals[SERVER_DISPLAY], 0, TRUE);
+		g_signal_emit(server, signals[SERVER_DISPLAY], 0, timestamp, TRUE);
 		return TRUE;
 	}
 
@@ -1251,7 +1339,7 @@ show_indicator_to_user (IndicateServer * server, guint id, GError ** error)
 		return FALSE;
 	}
 
-	indicate_indicator_user_display(indicator);
+	indicate_indicator_user_display(indicator, timestamp);
 	return TRUE;
 }
 
@@ -1297,7 +1385,7 @@ indicator_displayed (IndicateServer * server, gchar * sender, guint id, gboolean
 
 /* Virtual Functions */
 gboolean 
-_indicate_server_get_indicator_count (IndicateServer * server, guint * count, GError **error)
+_indicate_interface_server_get_indicator_count (IndicateServer * server, guint * count, GError **error)
 {
 	IndicateServerClass * class = INDICATE_SERVER_GET_CLASS(server);
 
@@ -1318,7 +1406,7 @@ _indicate_server_get_indicator_count (IndicateServer * server, guint * count, GE
 }
 
 gboolean 
-_indicate_server_get_indicator_list (IndicateServer * server, GArray ** indicators, GError ** error)
+_indicate_interface_server_get_indicator_list (IndicateServer * server, GArray ** indicators, GError ** error)
 {
 	IndicateServerClass * class = INDICATE_SERVER_GET_CLASS(server);
 
@@ -1339,7 +1427,7 @@ _indicate_server_get_indicator_list (IndicateServer * server, GArray ** indicato
 }
 
 gboolean 
-_indicate_server_get_indicator_property (IndicateServer * server, guint id, gchar * property, gchar ** value, GError **error)
+_indicate_interface_server_get_indicator_property (IndicateServer * server, guint id, gchar * property, GValue * value, GError **error)
 {
 	IndicateServerClass * class = INDICATE_SERVER_GET_CLASS(server);
 
@@ -1360,7 +1448,7 @@ _indicate_server_get_indicator_property (IndicateServer * server, guint id, gcha
 }
 
 gboolean 
-_indicate_server_get_indicator_property_group (IndicateServer * server, guint id, GPtrArray * properties, gchar *** value, GError **error)
+_indicate_interface_server_get_indicator_property_group (IndicateServer * server, guint id, GPtrArray * properties, gchar *** value, GError **error)
 {
 	IndicateServerClass * class = INDICATE_SERVER_GET_CLASS(server);
 
@@ -1381,7 +1469,7 @@ _indicate_server_get_indicator_property_group (IndicateServer * server, guint id
 }
 
 gboolean 
-_indicate_server_get_indicator_properties (IndicateServer * server, guint id, gchar *** properties, GError **error)
+_indicate_interface_server_get_indicator_properties (IndicateServer * server, guint id, gchar *** properties, GError **error)
 {
 	IndicateServerClass * class = INDICATE_SERVER_GET_CLASS(server);
 
@@ -1402,12 +1490,12 @@ _indicate_server_get_indicator_properties (IndicateServer * server, guint id, gc
 }
 
 gboolean 
-_indicate_server_show_indicator_to_user (IndicateServer * server, guint id, GError ** error)
+_indicate_interface_server_show_indicator_to_user (IndicateServer * server, guint id, guint timestamp, GError ** error)
 {
 	IndicateServerClass * class = INDICATE_SERVER_GET_CLASS(server);
 
 	if (class != NULL && class->show_indicator_to_user != NULL) {
-		return class->show_indicator_to_user (server, id, error);
+		return class->show_indicator_to_user (server, id, timestamp, error);
 	}
 
 	if (error) {
@@ -1425,7 +1513,7 @@ _indicate_server_show_indicator_to_user (IndicateServer * server, guint id, GErr
 /* DBus function to wrap a virtual function call so that
    it can be subclassed if someone so chooses */
 gboolean
-_indicate_server_indicator_displayed (IndicateServer * server, guint id, gboolean displayed, DBusGMethodInvocation * method)
+_indicate_interface_server_indicator_displayed (IndicateServer * server, guint id, gboolean displayed, DBusGMethodInvocation * method)
 {
 	IndicateServerClass * class = INDICATE_SERVER_GET_CLASS(server);
 
@@ -1500,7 +1588,7 @@ interest_string_to_enum (gchar * interest_string)
 }
 
 gboolean
-_indicate_server_show_interest (IndicateServer * server, gchar * interest, DBusGMethodInvocation * method)
+_indicate_interface_server_show_interest (IndicateServer * server, gchar * interest, DBusGMethodInvocation * method)
 {
 	IndicateServerClass * class = INDICATE_SERVER_GET_CLASS(server);
 
@@ -1533,7 +1621,7 @@ _indicate_server_show_interest (IndicateServer * server, gchar * interest, DBusG
 }
 
 gboolean
-_indicate_server_remove_interest (IndicateServer * server, gchar * interest, DBusGMethodInvocation * method)
+_indicate_interface_server_remove_interest (IndicateServer * server, gchar * interest, DBusGMethodInvocation * method)
 {
 	IndicateServerClass * class = INDICATE_SERVER_GET_CLASS(server);
 
@@ -1566,7 +1654,7 @@ _indicate_server_remove_interest (IndicateServer * server, gchar * interest, DBu
 }
 
 gboolean
-_indicate_server_set_max_indicators (IndicateServer * server, gint max, DBusGMethodInvocation * method)
+_indicate_interface_server_set_max_indicators (IndicateServer * server, gint max, DBusGMethodInvocation * method)
 {
 	IndicateServerClass * class = INDICATE_SERVER_GET_CLASS(server);
 
@@ -1644,6 +1732,43 @@ indicate_server_get_max_indicators (IndicateServer * server)
 	return -1;
 }
 
+/**
+	indicate_server_set_menu:
+	@server: The #IndicateServer to use
+	@menu: A #DbusmenuServer object represting the menu
+
+	This function sets the internal menu representation for this
+	indicator.  It makes it so that when clients ask for information
+	on the menus it'll repospond with the address of this menu.
+
+	It is important to note that there is not signal about menus
+	changing.  So if the indicator is already visible there is no
+	way that a listener would know about the change, and thus is
+	likely to have the wrong menu (or no menu).
+
+	Return value: None.
+*/
+void
+indicate_server_set_menu (IndicateServer * server, DbusmenuServer * menu)
+{
+	g_return_if_fail(INDICATE_IS_SERVER(server));
+	IndicateServerPrivate * priv = INDICATE_SERVER_GET_PRIVATE(server);
+
+	if (priv->visible) {
+		g_warning("Menu being changed when the indicator is visible.  Listeners will NOT be notified of this change.");
+	}
+
+	if (priv->dbusmenu != NULL) {
+		g_object_unref(priv->dbusmenu);
+		priv->dbusmenu = NULL;
+	}
+
+	priv->dbusmenu = menu;
+	g_object_ref(priv->dbusmenu);
+
+	return;
+}
+
 /* *** Folks stuff *** */
 
 /* This checks for folk by looking at the sender value
